function [aucsumFinal, eersumFinal, apsumFinal] = vcdteval(evalFile, gtFile)
%
% [aucsumFinal, eersumFinal, apsumFinal] = vcdteval(evalFile, gtFile)
% Function initializes the computation of the Area under Curve (AUC), Equal Error Rate (EER) and
% the interpolated Average Precision (AP) from the confidence submissions to the VCDT 2011. 
%
% References:
% this script is adapted from the EER/AUC script from the VCDT 2008
% http://www.imageclef.org/system/files/evaltool.tgz
%
%
% Input:
%       evalFile:		the file to be evaluated. Please note that only the
%                       confidence part of the submission file should be
%                       used here, as AUC, EER and AP require confidence
%                       predictions
%       gtFile:			the GT file of the VCDT 2011
%
% Output:
%       aucsumFinal:	mean AUC
%       eersumFinal:	mean EER
%       apsumFinal:     mean AP
%


N_CLASSES=99;

f= fopen(gtFile);
ok=1;
GT={};
count=1;
while ok
  [fname,ok]=fscanf(f,'%s',1);
  if ok
   probs=fscanf(f,'%f',N_CLASSES);
   GT{count,1}=fname;
   GT{count,2}=probs;
   count=count+1;
 end
end
fclose(f);


f=fopen(evalFile);
ok=1;
SUB={};
count=1;
while ok
  [fname,ok]=fscanf(f,'%s',1);
  if ok
   probs=fscanf(f,'%f',N_CLASSES);
   SUB{count,1}=fname;
   SUB{count,2}=probs;
   count=count+1;
  end
end
fclose(f);

f=fopen('concepts.txt');
a=fscanf(f,'%s',2);
classes=cell(N_CLASSES,1);
for c=1:N_CLASSES
  a=fscanf(f,'%i',1);
  a=fscanf(f,'%s',1);
  classes{c}=a;
end
fclose(f);

curves=cell(N_CLASSES,1);
eers=cell(N_CLASSES,1);
aucs=cell(N_CLASSES,1);
aps=cell(N_CLASSES,1);
leg=cell(N_CLASSES,1);

fullscreen = get(0,'ScreenSize');
figure('Position',[0 -50 fullscreen(3) fullscreen(4)])
%clf
hold on;
aucsum=0;
eersum=0;
apsum=0;
xlabel('false positive rate')
ylabel('true positive rate')

%sort both files
[B,IX] = sort(GT(:,1));
GT(:,1) = B;
tmp = GT(:,2);
GT(:,2) = tmp(IX(:,1),:);
[B2, IX2] = sort(SUB(:,1));
SUB(:,1) = B2;
tmp = SUB(:,2);
SUB(:,2) = tmp(IX2(:,1),:);


ColorSet = [
    0,1,0;
    1,1,0;
    0,1,1;
    0.7,0,0;
    0,0,1;
    0.5,0,1;
    1,0,1;
    0.5,0,0.5;
    1,0,0;
    0,0,0;
    ];

fid = fopen(strcat(evalFile,'_results.txt'), 'wt');

for cls=1:N_CLASSES
  a=zeros(size(GT,1),2);
  for n=1:size(GT,1)
   if  strcmp(GT{n,1},SUB{n,1})
     a(n,2)=GT{n,2}(cls);
     a(n,1)=SUB{n,2}(cls); 
   else
      error('File IDs are not the same');
   end
  end

  fprintf(1,'Class: %2i %20s ',cls,classes{cls});
  [curve,auc,eer,ap]=roc(a);
  fprintf(1,' AP: %f, EER: %f, AUC: %f\n',ap, eer,auc);
  
  fprintf(fid, 'Class: %2i %20s AP: %f, EER: %f, AUC: %f\n',cls,classes{cls},ap, eer,auc);
  
  curves{cls}=curve; 
  eers{cls}=eer;
  aucs{cls}=auc;
  aps{cls}=ap;

  aucsum=aucsum+auc;
  eersum=eersum+eer;
  apsum = apsum+ap;

  leg{cls,1}=sprintf('%15s AP: %f EER: %f AUC: %f',classes{cls},aps{cls},eers{cls},aucs{cls});
end
fprintf(1,'Summary                        MAP: %f, EER: %f, AUC:%f\n',apsum/N_CLASSES, eersum/N_CLASSES, aucsum/N_CLASSES);
fprintf(fid,'Summary                        MAP: %f,EER: %f, AUC:%f\n',apsum/N_CLASSES, eersum/N_CLASSES, aucsum/N_CLASSES);
fclose(fid);

hold on
for i=1:10
    plot(curves{i}(:,1),curves{i}(:,2),'-','Color', ColorSet(i,:));
end
for i=11:20
    plot(curves{i}(:,1),curves{i}(:,2),'-.','Color', ColorSet(i-10,:));
end
title(sprintf('%s, MAP: %f, EER: %f, AUC: %f', evalFile, apsum/N_CLASSES, eersum/N_CLASSES, aucsum/N_CLASSES))
legend(leg(1:20),'location', 'NorthEastOutside');%, 'boxoff');
set(gcf,'PaperPositionMode','auto')
set(gcf,'PaperUnits','normalized')
orient(gcf, 'landscape')
print(sprintf('%s_1.ps',evalFile),'-depsc2')
saveas(gcf,sprintf('%s_1.fig',evalFile))
hold off


figure('Position',[0 -50 fullscreen(3) fullscreen(4)])
hold on
for i=21:30
    plot(curves{i}(:,1),curves{i}(:,2),'-','Color', ColorSet(i-20,:));
end
for i=31:40
    plot(curves{i}(:,1),curves{i}(:,2),'-.','Color', ColorSet(i-30,:));
end
title(sprintf('%s,MAP: %f, EER: %f, AUC: %f', evalFile, apsum/N_CLASSES, eersum/N_CLASSES, aucsum/N_CLASSES))
legend(leg(21:40),'location', 'NorthEastOutside');%, 'boxoff');
set(gcf,'PaperPositionMode','auto')
set(gcf,'PaperUnits','normalized')
orient(gcf, 'landscape')
print(sprintf('%s_2.ps',evalFile),'-depsc2')
saveas(gcf,sprintf('%s_2.fig',evalFile))
hold off


figure('Position',[0 -50 fullscreen(3) fullscreen(4)])
hold on
for i=41:50
    plot(curves{i}(:,1),curves{i}(:,2),'-','Color', ColorSet(i-40,:));
end
for i=51:60
    plot(curves{i}(:,1),curves{i}(:,2),'-.','Color', ColorSet(i-50,:));
end
title(sprintf('%s, MAP: %f, EER: %f, AUC: %f', evalFile, apsum/N_CLASSES, eersum/N_CLASSES, aucsum/N_CLASSES))
legend(leg(41:60),'location', 'NorthEastOutside');%, 'boxoff');
set(gcf,'PaperPositionMode','auto')
set(gcf,'PaperUnits','normalized')
orient(gcf, 'landscape')
print(sprintf('%s_3.ps',evalFile),'-depsc2')
saveas(gcf,sprintf('%s_3.fig',evalFile))
hold off


figure('Position',[0 -50 fullscreen(3) fullscreen(4)])
hold on
for i=61:70
    plot(curves{i}(:,1),curves{i}(:,2),'-','Color', ColorSet(i-60,:));
end
for i=71:80
    plot(curves{i}(:,1),curves{i}(:,2),'-.','Color', ColorSet(i-70,:));
end
title(sprintf('%s, MAP: %f, EER: %f, AUC: %f', evalFile, apsum/N_CLASSES, eersum/N_CLASSES, aucsum/N_CLASSES))
legend(leg(61:80),'location', 'NorthEastOutside');%, 'boxoff');
set(gcf,'PaperPositionMode','auto')
set(gcf,'PaperUnits','normalized')
orient(gcf, 'landscape')
print(sprintf('%s_4.ps',evalFile),'-depsc2')
saveas(gcf,sprintf('%s_4.fig',evalFile))
hold off


figure('Position',[0 -50 fullscreen(3) fullscreen(4)])
hold on
for i=81:90
    plot(curves{i}(:,1),curves{i}(:,2),'-','Color', ColorSet(i-80,:));
end
for i=91:99
    plot(curves{i}(:,1),curves{i}(:,2),'-.','Color', ColorSet(i-90,:));
end
title(sprintf('%s, MAP: %f, EER: %f, AUC: %f', evalFile, apsum/N_CLASSES, eersum/N_CLASSES, aucsum/N_CLASSES))
legend(leg(81:99),'location', 'NorthEastOutside');%, 'boxoff');
set(gcf,'PaperPositionMode','auto')
set(gcf,'PaperUnits','normalized')
orient(gcf, 'landscape')
print(sprintf('%s_5.ps',evalFile),'-depsc2')
saveas(gcf,sprintf('%s_5.fig',evalFile))
hold off

aucsumFinal = aucsum/N_CLASSES;
eersumFinal = eersum/N_CLASSES;
apsumFinal = apsum/N_CLASSES;
close all;
